const Mesa = require('vendor/Mesa.js');
const ShopifyGraphql = require('vendor/ShopifyGraphql.js');
const Shopify= require('vendor/Shopify.js');

/**
 * A Mesa Script exports a class with a script() method.
 */
module.exports = new class {
  script = (payload, context) => {
    let vars = context.steps;

    let existing = this.getExisting(vars);
    if (existing) {
      this.updateRow(payload, vars, existing);
    } else {
      this.createRow(payload, vars);
    }

    Mesa.output.next({"changeme": true});
  }

  getExisting = (vars) => {
    let rowObjects = this.getRows(vars);

    for (let rowObject of rowObjects) {
      if (rowObject.fields['File ID'] == payload['File ID']) {
        return rowObject;
      }
    }

    return null;    
  }

  getRows = (vars) => {
    let credential = JSON.parse(Mesa.credential.get('googlesheets'));

    let options = {
      "headers": {
        "Content-Type": "application\/json",
        "Authorization": "Bearer " + credential.access_token,
      },
      "query": {
        "majorDimension": "ROWS",
        "valueRenderOption": "UNFORMATTED_VALUE",
        "dateTimeRenderOption": "FORMATTED_STRING"
      }
    }

    let url = "https://sheets.googleapis.com/v4/spreadsheets/1zGa0Lge4xGmOsMvKewdA6Ip9ojwCLx1rD1LR-CKlOx4/values/'Images'!A1:Z200"
    let results = Mesa.request.get(url, options);

    Mesa.log.info("results", results.values);

    let rowObjects = [];
    let firstRow = results.values[0];
    for (let i = 1; i < results.values.length; i++) {
      let rowObject = {
        "row": i + 1,
        "fields": []
      };
      Mesa.log.info("i: " + i);
      let row = results.values[i];
      for (let j = 0; j < row.length; j++) {
        rowObject.fields[firstRow[j]] = row[j];
      }

      rowObjects.push(rowObject);
    }

    Mesa.log.info("row objects", rowObjects);
  
    return rowObjects;
  }

  createRow = (vars) => {
    let credential = JSON.parse(Mesa.credential.get('googlesheets'));

    let options = {
      "headers": {
        "Content-Type": "application\/json",
        "Authorization": "Bearer " + credential.access_token,
      },
      "query": {
        "valueInputOption": "USER_ENTERED",
      },
    };

    let data = {
      "range": "'Images'!A1:Z200",
      "majorDimension": "ROWS",
      "values": [Object.values(payload)]
    };

    let url = "https://sheets.googleapis.com/v4/spreadsheets/1zGa0Lge4xGmOsMvKewdA6Ip9ojwCLx1rD1LR-CKlOx4/values/'Images'!A1:Z200:append";
    let results = Mesa.request.post(url, data, options);

    return results;
  }

  updateRow = (payload, vars, rowObject) => {
    let credential = JSON.parse(Mesa.credential.get('googlesheets'));

    let rowNumber = rowObject.row;

    let options = {
      "headers": {
        "Content-Type": "application\/json",
        "Authorization": "Bearer " + credential.access_token,
      },
      "query": {
        "valueInputOption": "USER_ENTERED",
      },
    };

    let data = {
      "range": `'Images'!A${rowNumber}:Z${rowNumber}`,
      "majorDimension": "ROWS",
      "values": [Object.values(payload)],
    };

    let url = `https://sheets.googleapis.com/v4/spreadsheets/${this.sheetId()}/values/'Images'!A${rowNumber}:Z${rowNumber}`;
    let results = Mesa.request.put(url, data, options);

    return results;

  }

  sheetId = (vars) => {
    return '1zGa0Lge4xGmOsMvKewdA6Ip9ojwCLx1rD1LR-CKlOx4';
  }
}